<?php

/**
 * A base model to provide the basic CRUD
 * actions for all models that inherit from it.
 *
 * @package CodeIgniter
 * @subpackage MY_Model
 * @license GPLv3 <http://www.gnu.org/licenses/gpl-3.0.txt>
 * @link http://github.com/philsturgeon/codeigniter-base-model
 * @version 1.4
 * @author Jamie Rumbelow <http://jamierumbelow.net>
 * @modified Phil Sturgeon <http://philsturgeon.co.uk>
 * @modified Dan Horrigan <http://dhorrigan.com>
 * @modified Nickolas Sharp <http://myshuitings.com>
 * @copyright Copyright (c) 2009, Jamie Rumbelow <http://jamierumbelow.net>
 */
class MY_Model extends CI_Model {
  /**
   * The database table to use, only
   * set if you want to bypass the magic
   *
   * @var string
   */
  protected $_table;

  /**
   * The primary key, by default set to
   * `id`, for use in some functions.
   *
   * @var string
   */
  protected $primary_key = 'id';

  /**
   * An array of functions to be called before
   * a record is created.
   *
   * @var array
   */
  protected $before_create = array();

  /**
   * An array of functions to be called after
   * a record is created.
   *
   * @var array
   */
  protected $after_create = array();

  /**
   * An array of validation rules
   *
   * @var array
   */
  protected $before_update = array();

  /**
   * An array of functions to be called after
   * a record is created.
   *
   * @var array
   */
  protected $after_update = array();

  /**
   * An array of validation rules
   *
   * @var array
   */
  protected $before_delete = array();

  /**
   * An array of functions to be called after
   * a record is created.
   *
   * @var array
   */
  protected $after_delete = array();

  /**
   * An array of validation rules
   *
   * @var array
   */
  protected $validate = array();

  /**
   * Skip the validation
   *
   * @var bool
   */
  protected $skip_validation = FALSE;

  protected $order_field = "menu_weight";
  protected $live_field = "is_active";
  protected $condition_field = "is_deleted";
  protected $attribute_fields = array('menu_destination');
  protected $home_mode = "1";

  /**
   * Wrapper to __construct for when loading
   * class is a superclass to a regular controller,
   * i.e. - extends Base not extends Controller.
   *
   * @return void
   * @author Jamie Rumbelow
   */
  public function MY_Model() {
    $this->__construct();
  }

  public function _initialize() {
    $args =& func_get_args();
    foreach ($args as $arg) {
      foreach ($arg as $key => $value) {
        $this->$key = $value;
      }
    }
  }
  
  /**
   * The class constructer, tries to guess
   * the table name.
   *
   * @author Jamie Rumbelow
   */
  public function __construct() {
    parent::__construct();
    $this->load->helper('inflector');
    $this->_fetch_table();
  }

  /**
   * Get a single record by creating a WHERE clause with
   * a value for your primary key
   *
   * @param string $primary_value The value of your primary key
   * @return object
   * @author Phil Sturgeon
   */
  public function _get($primary_value) {
    return $this->db->where($this->primary_key, $primary_value)
      ->get($this->_table)
      ->row();
  }

  /**
   * Get a single record by creating a WHERE clause with
   * the key of $key and the value of $val.
   *
   * @param string $key The key to search by
   * @param string $val The value of that key
   * @return object
   * @author Phil Sturgeon
   */
  public function _get_by() {
    $where =& func_get_args();
    $this->_set_where($where);

    return $this->db->get($this->_table)
      ->row();
  }

  /**
   * Similar to get_by(), but returns a result array of
   * many result objects.
   *
   * @param string $primary_value The value of your primary key
   * @return array
   * @author Phil Sturgeon
   */
  public function _get_many($primary_value) {
    $this->db->where($this->primary_key, $primary_value);
    return $this->_get_all();
  }

  /**
   * Similar to get_by(), but returns a result array of
   * many result objects.
   *
   * @param string $key The key to search by
   * @param string $val The value of that key
   * @return array
   * @author Phil Sturgeon
   */
  public function _get_many_by() {
    $where =& func_get_args();
    $this->_set_where($where);

    return $this->_get_all();
  }

  /**
   * Get all records in the database
   *
   * @return array
   * @author Jamie Rumbelow
   */
  public function _get_all() {
    return $this->db->get($this->_table)
      ->result();
  }

  /**
   * Similar to get_by(), but returns a result array of
   * many result objects.
   *
   * @param string $key The key to search by
   * @param string $val The value of that key
   * @return array
   * @author Phil Sturgeon
   */
  public function _count_by() {
    $where =& func_get_args();
    $this->_set_where($where);

    return $this->db->count_all_results($this->_table);
  }

  /**
   * Get all records in the database
   *
   * @return array
   * @author Phil Sturgeon
   */
  public function _count_all() {
    return $this->db->count_all($this->_table);
  }

  /**
   * Insert a new record into the database,
   * calling the before and after create callbacks.
   * Returns the insert ID.
   *
   * @param array $data Information
   * @return integer
   * @author Jamie Rumbelow
   * @modified Dan Horrigan
   */
  public function _insert($data, $skip_validation = FALSE) {
    $valid = TRUE;
    if($skip_validation === FALSE) {
      $valid = $this->_run_validation($data);
    }

    if($valid) {
      $data = $this->_run_before_create($data);
      $this->db->insert($this->_table, $data);
      $this->_run_after_create($data, $this->db->insert_id());

      $this->skip_validation = FALSE;
      return $this->db->insert_id();
    }
    else {
      return FALSE;
    }
  }

  /**
   * Similar to insert(), just passing an array to insert
   * multiple rows at once. Returns an array of insert IDs.
   *
   * @param array $data Array of arrays to insert
   * @return array
   * @author Jamie Rumbelow
   */
  public function _insert_many($data, $skip_validation = FALSE) {
    $ids = array();

    foreach ($data as $row) {
      $valid = TRUE;
      if($skip_validation === FALSE) {
        $valid = $this->_run_validation($data);
      }

      if($valid) {
        $data = $this->_run_before_create($row);
        $this->db->insert($this->_table, $row);
        $this->_run_after_create($row, $this->db->insert_id());

        $ids[] = $this->db->insert_id();
      }
      else {
        $ids[] = FALSE;
      }
    }

    $this->skip_validation = FALSE;
    return $ids;
  }

  /**
   * Update a record, specified by an ID.
   *
   * @param integer $id The row's ID
   * @param array $array The data to update
   * @return bool
   * @author Jamie Rumbelow
   */
  public function _update($primary_value, $data, $skip_validation = FALSE) {
    $valid = TRUE;
    if($skip_validation === FALSE) {
      $valid = $this->_run_validation($data);
    }
    if($valid) {
      $new_data = $this->_run_before_update($data, $this->primary_key, $primary_value);
      if($new_data) {
        $data = (object) array_merge((array) $new_data, (array) $data);
        $this->db->where($this->primary_key, $primary_value)
          ->set($data)
          ->update($this->_table);
        $this->_run_after_update($data, $this->primary_key, $primary_value);
        $this->skip_validation = FALSE;
        return TRUE;
      }
      return FALSE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Update a record, specified by $key and $val.
   *
   * @param string $key The key to update with
   * @param string $val The value
   * @param array $array The data to update
   * @param bool $skip_validation
   * @return bool
   * @author Jamie Rumbelow
   */
  public function _update_by() {
    $args =& func_get_args();
    $skip_validation = array_pop($args);
    $data = array_pop($args);
    $this->_set_where($args);
    $matches = $this->_get_many_by($args[0], $args[1]);
    foreach($matches as $match) {
      $this->_update($match->id, $data, $skip_validation);
    }
    return TRUE;
  }

  /**
   * Updates many records, specified by an array
   * of IDs.
   *
   * @param array $primary_values The array of IDs
   * @param array $data The data to update
   * @return bool
   * @author Phil Sturgeon
   */
  public function _update_many($primary_values, $data, $skip_validation = FALSE) {
    foreach ($primary_values as $primary_value) {
      $this->_update($primary_value, $data, $skip_validation);
    }
    return TRUE;
  }

  /**
   * Updates all records, by cycling the update function
   * (very innefective but the best way to do the revisioning at the moment)
   * other options would be get all, then update the values in the array, then do an update all normal
   * then do the insert many into the rev table... but since I don't see using update all pretty much ever
   * its not an issue for now.
   *
   * @param array $data The data to update
   * @return bool
   * @since 1.1.3
   * @author Phil Sturgeon
   */
  public function _update_all($data, $skip_validation = FALSE) {
    $rows = $this->_get_all();
    foreach($rows as $row) {
      $this->_update($row->id, $data, $skip_validation);
    }
    return TRUE;
  }

  /**
   * Delete a row from the database table by the
   * ID.
   *
   * @param integer $id
   * @return bool
   * @author Jamie Rumbelow
   */
  public function _delete($id) {
    $new_data = $this->_run_before_delete($this->primary_key, $id);
    if($new_data) {
      if($new_data->is_active == 1) {
        $this->_deactivate($this->primary_key, $id);
      }
      elseif($new_data->is_deleted == 0) {
        $this->_soft_delete($this->primary_key, $id);
      }
      else {
        $this->db->where($this->primary_key, $id)
          ->delete($this->_table);
        $this->_run_after_delete($new_data, $this->primary_key, $id);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Delete a row from the database table by the
   * key and value.
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Phil Sturgeon
   */
  public function _delete_by() {
    $where =& func_get_args();
    $new_data = $this->_run_before_delete($where[0], $where[1]);
    if($new_data) {
      if($new_data->is_active == 1) {
        $this->_deactivate($where[0], $where[1]);
      }
      elseif($new_data->is_deleted == 0) {
        $this->_soft_delete($where[0], $where[1]);
      }
      else {
        $this->_set_where($where);
        $this->db->delete($this->_table);
        $this->_run_after_delete($new_data, $where[0], $where[1]);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Delete many rows from the database table by
   * an array of IDs passed.
   *
   * @param array $primary_values
   * @return bool
   * @author Phil Sturgeon
   */
  public function _delete_many($primary_values) {
    foreach ($primary_values as $primary_value) {
      $new_data = $this->_run_before_delete($this->primary_key, $primary_value);
      if($new_data) {
        if($new_data->is_active == 1) {
          $this->_deactivate($this->primary_key, $primary_value);
        }
        elseif($new_data->is_deleted == 0) {
          $this->_soft_delete($this->primary_key, $primary_value);
        }
        else {
          $this->db->where($this->primary_key, $primary_value)
            ->delete($this->_table);
          $this->_run_after_delete($new_data, $this->primary_key, $primary_value);
        }
      }
    }
    return TRUE;
  }

  /**
   * Deactivate the content by setting is_active = 0
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Nickolas Sharp
   */
  public function _deactivate() {
    $data = array('is_active' => '0');
    $args = func_get_args();
    if(count($args) == 1) {
      //then we check if they passed the id, or an array of id's
      if(is_array($args[0])) {
        //then they passed an array of ids and we call _update_many
        return $this->_update_many($args[0], $data, TRUE);
      }
      else {
        //then they just passed the id and we call _update
        return $this->_update($args[0], $data, TRUE);
      }
    }
    else {
      //then they passed $key and $value and we call update_by
      return $this->_update_by($args[0], $args[1], $data, TRUE);
    }
  }

  /**
   * Reactivate the content by setting is_active = 1
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Nickolas Sharp
   */
  public function _reactivate() {
    $data = array('is_active' => '1');
    $args = func_get_args();
    if(count($args) == 1) {
      //then we check if they passed the id, or an array of id's
      if(is_array($args[0])) {
        //then they passed an array of ids and we call _update_many
        return $this->_update_many($args[0], $data, TRUE);
      }
      else {
        //then they just passed the id and we call _update
        return $this->_update($args[0], $data, TRUE);
      }
    }
    else {
      //then they passed $key and $value and we call update_by
      return $this->_update_by($args[0], $args[1], $data, TRUE);
    }
  }

  /**
   * Soft Delete the content by setting is_deleted = 1
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Nickolas Sharp
   */
  public function _soft_delete() {
    $data = array('is_deleted' => '1');
    $args = func_get_args();
    if(count($args) == 1) {
      //then we check if they passed the id, or an array of id's
      if(is_array($args[0])) {
        //then they passed an array of ids and we call _update_many
        return $this->_update_many($args[0], $data, TRUE);
      }
      else {
        //then they just passed the id and we call _update
        return $this->_update($args[0], $data, TRUE);
      }
    }
    else {
      //then they passed $key and $value and we call update_by
      return $this->_update_by($args[0], $args[1], $data, TRUE);
    }
  }

  /**
   * Restore from Soft Delete the content by setting is_deleted = 0
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Nickolas Sharp
   */
  public function _soft_delete_restore() {
    $data = array('is_deleted' => '0');
    $args = func_get_args();
    if(count($args) == 1) {
      //then we check if they passed the id, or an array of id's
      if(is_array($args[0])) {
        //then they passed an array of ids and we call _update_many
        return $this->_update_many($args[0], $data, TRUE);
      }
      else {
        //then they just passed the id and we call _update
        return $this->_update($args[0], $data, TRUE);
      }
    }
    else {
      //then they passed $key and $value and we call update_by
      return $this->_update_by($args[0], $args[1], $data, TRUE);
    }
  }
  /**
   * Revision the content without changing anything
   *
   * @param string $key
   * @param string $value
   * @return bool
   * @author Nickolas Sharp
   */
  public function _revision() {
    $data = '';
    $args = func_get_args();
    if(count($args) == 1) {
      //then we check if they passed the id, or an array of id's
      if(is_array($args[0])) {
        //then they passed an array of ids and we call _update_many
        return $this->_update_many($args[0], $data, TRUE);
      }
      else {
        //then they just passed the id and we call _update
        return $this->_update($args[0], $data, TRUE);
      }
    }
    else {
      //then they passed $key and $value and we call update_by
      return $this->_update_by($args[0], $args[1], $data, TRUE);
    }
  }

  function dropdown() {
    $args =& func_get_args();

    if(count($args) == 2) {
      list($key, $value) = $args;
    }

    else {
      $key = $this->primary_key;
      $value = $args[0];
    }

    $query = $this->db->select(array($key, $value))
      ->get($this->_table);

    $options = array();
    foreach ($query->result() as $row) {
      $options[$row->{$key}] = $row->{$value};
    }

    return $options;
  }

  /**
   * Orders the result set by the criteria,
   * using the same format as CI's AR library.
   *
   * @param string $criteria The criteria to order by
   * @return void
   * @since 1.1.2
   * @author Jamie Rumbelow
   */
  public function order_by($criteria, $order = 'ASC') {
    $this->db->order_by($criteria, $order);
    return $this;
  }

  /**
   * Limits the result set by the integer passed.
   * Pass a second parameter to offset.
   *
   * @param integer $limit The number of rows
   * @param integer $offset The offset
   * @return void
   * @since 1.1.1
   * @author Jamie Rumbelow
   */
  public function limit($limit, $offset = 0) {
    $this->db->limit($limit, $offset);
    return $this;
  }

  /**
   * Tells the class to skip the insert validation
   *
   * @return void
   * @author Dan Horrigan
   */
  public function skip_validation() {
    $this->skip_validation = TRUE;
    return $this;
  }
  /**
   * Runs the before create actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Jamie Rumbelow
   */
  private function _run_before_create() {
    $data = func_get_args();
    $data = $data[0];

    foreach ($this->before_create as $method) {
      $data = func_get_args();
      $data = call_user_func_array(array($this, $method), $data);
    }

    return $data;
  }

  /**
   * Runs the after create actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Jamie Rumbelow
   */
  private function _run_after_create() {
    $data = func_get_args();
    foreach ($this->after_create as $method) {
      call_user_func_array(array($this, $method), $data);
    }
  }

  /**
   * Runs the before update actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Nickolas Sharp
   */
  private function _run_before_update() {
    $data = func_get_args();
    $data = $data[0];

    foreach ($this->before_update as $method) {
      $data = func_get_args();
      $data = call_user_func_array(array($this, $method), $data);
    }

    return $data;
  }

  /**
   * Runs the after update actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Nickolas Sharp
   */
  private function _run_after_update() {
    $data = func_get_args();
    foreach ($this->after_update as $method) {
      call_user_func_array(array($this, $method), $data);
    }
  }

  /**
   * Runs the before delete actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Nickolas Sharp
   */
  private function _run_before_delete() {
    $data = func_get_args();
    foreach ($this->before_delete as $method) {
      $data = call_user_func_array(array($this, $method), $data);
    }

    return $data;
  }

  /**
   * Runs the after delete actions.
   *
   * @param array $data The array of actions
   * @return void
   * @author Nickolas Sharp
   */
  private function _run_after_delete() {
    $data = func_get_args();
    foreach ($this->after_delete as $method) {
      call_user_func_array(array($this, $method), $data);
    }
  }

  /**
   * Runs validation on the passed data.
   *
   * @return bool
   * @author Dan Horrigan
   */
  private function _run_validation($data) {
    if($this->skip_validation) {
      return TRUE;
    }
    if(!empty($this->validate)) {
      foreach($data as $key => $val) {
        $_POST[$key] = $val;
      }
      $this->load->library('form_validation');
      if(is_array($this->validate)) {
        $this->form_validation->set_rules($this->validate);
        return $this->form_validation->run();
      }
      else {
        $this->form_validation->run($this->validate);
      }
    }
    else {
      return TRUE;
    }
  }

  /**
   * Fetches the table from the pluralised model name.
   *
   * @return void
   * @author Jamie Rumbelow
   */
  private function _fetch_table() {
    if ($this->_table == NULL) {
      $class = preg_replace('/(_m|_model)?$/', '', get_class($this));

      $this->_table = plural(strtolower($class));
    }
  }

  /**
   * Sets where depending on the number of parameters
   *
   * @return void
   * @author Phil Sturgeon
   */
  private function _set_where($params) {
    if(count($params) == 1) {
      $this->db->where($params[0]);
    }

    else {
      $this->db->where($params[0], $params[1]);
    }
  }

  /**
   * Get Head
   *
   * @return array
   * Use: takes a parent_id as an input and outputs that item
   */
  public function _get_head($id, $table = 0) {
    $table = ($table === 0)?$this->_table:$table;
    $this->db->where($table.'_id', $id);
    if ($this->condition_field) {
      $this->db->where($this->condition_field, '0');
    }
    $this->db->order_by($this->order_field, "asc");
    $query = $this->db->get($table);
    return $query->result_array();
  }
  /**
   * Get Items
   *
   * @return array
   * Use: takes a parent_id as an input and outputs all the children
   */
  public function & _get_items($id = 0, $table = 0) {
    $table = ($table === 0)?$this->_table:$table;
    $itemOverview =& $this->_get_head($id, $table);
    foreach ($itemOverview as $key => $val) {
      $item = $itemOverview[$key];
      if ($item['id'] != 0) {
        $childItems =& $this->_get_items($item['id'], $table);
      }
      if (!empty($childItems)) {
        $item['children'] =& $childItems;
      }
      $items[] = $item;
    }
    return $items;
  }
}
